==> oer-0.0.1-20190731.023331-5-sources.jar: com/xdja/pki/itsca/oer/cert/CertificateBuilder.java <==
package com.xdja.pki.itsca.oer.cert;


import com.xdja.pki.itsca.oer.asn1.*;
import com.xdja.pki.itsca.oer.asn1.base.Enumerated;
import com.xdja.pki.itsca.oer.asn1.base.Null;
import com.xdja.pki.itsca.oer.utils.BCUtils;
import com.xdja.pki.itsca.oer.utils.ByteArrayUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.security.PrivateKey;
import java.security.PublicKey;

public class CertificateBuilder {

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    private SequenceOfItsAidList itsAidList;
    private SequenceOfItsAidSspList itsAidSspList;

    private GeographicRegion geographicRegion;

    public void setItsAidList(SequenceOfItsAidList itsAidList) {
        this.itsAidList = itsAidList;
    }

    public void setItsAidSspList(SequenceOfItsAidSspList itsAidSspList) {
        this.itsAidSspList = itsAidSspList;
    }

    public void setGeographicRegion(GeographicRegion geographicRegion) {
        this.geographicRegion = geographicRegion;
    }

    /**
     * 构造 四边形区域
     *
     * @param northWestLatitude  北 西 维度
     * @param northWestLongitude 北 西 经度
     * @param southEastLatitude  南 东 纬度
     * @param southEastLongitude 南 东 经度
     * @return 四边形区域信息
     */
    public void setGeographicRegion(
            int northWestLatitude,
            int northWestLongitude,
            int southEastLatitude,
            int southEastLongitude
    ) {
        this.geographicRegion = new GeographicRegion();
        SequenceOfRectangularRegion sequenceOfRectangularRegion = new SequenceOfRectangularRegion();
        RectangularRegionBuilder rectangularRegionBuilder = new RectangularRegionBuilder();
        RectangularRegion rectangularRegion = rectangularRegionBuilder.build(northWestLatitude, northWestLongitude, southEastLatitude, southEastLongitude);
        sequenceOfRectangularRegion.addRectangularRegion(rectangularRegion);
        this.geographicRegion.setRectangularRegion(sequenceOfRectangularRegion);
    }

    public Certificate build(PrivateKey privateKey, PublicKey verifyPublicKey, PublicKey encPublicKey, String subjectName, byte[] issuedHashId, Enumerated.Value subjectTypeValue) throws IOException {

        TbsCertBuilder tbsCertBuilder = new TbsCertBuilder();

        if (itsAidList != null) {
            tbsCertBuilder.setItsAidList(itsAidList);
        }
        if (itsAidSspList != null) {
            tbsCertBuilder.setItsAidSspList(itsAidSspList);
        }
        if (geographicRegion != null) {
            tbsCertBuilder.setGeographicRegion(geographicRegion);
        }

        TbsCert tbsCert = tbsCertBuilder.build(verifyPublicKey, encPublicKey, subjectName, subjectTypeValue);

        return build(privateKey, issuedHashId, tbsCert);
    }

    public Certificate build(PrivateKey privateKey, byte[] issuedHashId, TbsCert tbsCert) throws IOException {
        Certificate certificate = new Certificate();
        ByteArrayUtils.printHexBinary(0, "Certificate SEQUENCE", null);

        ByteArrayUtils.printHexBinary(1, "Version Uint8 INTEGER", null);
        ByteArrayUtils.printHexBinary(1, "Version", ByteBuffer.allocate(2).putShort((short) 2).array());

        IssuerId issuerId = new IssuerId();
        if (issuedHashId != null) {
            ByteArrayUtils.printHexBinary(1, "IssuerId CHOICE [index = 1]", null);

            ByteArrayUtils.printHexBinary(2, "CertificateDigest SEQUENCE", null);
            CertificateDigest certificateDigest = new CertificateDigest();

            ByteArrayUtils.printHexBinary(3, "HashAlgorithm ENUMERATED", null);
            HashAlgorithm hashAlgorithm = new HashAlgorithm(HashAlgorithm.SGD_SM3);
            certificateDigest.setHashAlgorithm(hashAlgorithm);
            ByteArrayUtils.printHexBinary(3, "HashAlgorithm", hashAlgorithm.getEncode());

            ByteArrayUtils.printHexBinary(3, "HashedId8 OCTER STRING", null);
            HashedId8 hashedId8 = new HashedId8(issuedHashId);
            certificateDigest.setHashedId8(hashedId8);
            ByteArrayUtils.printHexBinary(3, "HashedId8", hashedId8.getEncode());

            ByteArrayUtils.printHexBinary(2, "CertificateDigest", certificateDigest.getEncode());
            issuerId.setCertificateDigest(certificateDigest);
        } else {
            ByteArrayUtils.printHexBinary(1, "IssuerId CHOICE [index = 0]", null);
            Null aNull = new Null();
            issuerId.setSelf(aNull);
        }
        certificate.setIssuerId(issuerId);
        ByteArrayUtils.printHexBinary(1, "IssuerId", issuerId.getEncode());

        ByteArrayUtils.printHexBinary(1, "TbsCert", null);
        certificate.setTbsCert(tbsCert);
        ByteArrayUtils.printHexBinary(1, "TbsCert", tbsCert.getEncode());

        byte[] tbsEncode = tbsCert.getEncode();
        byte[] r;
        byte[] s;
        try {
            byte[] sign;
//            if (privateKey instanceof SdfPrivateKey) {
//                sign = GMSSLSM2SignUtils.signBySdf(SdfCryptoType.YUNHSM, ((SdfPrivateKey) privateKey).getIndex(), new String(((SdfPrivateKey) privateKey).getPassword()), tbsEncode);
//            }else {
                sign = BCUtils.sm2SignByBC(privateKey, tbsEncode);
   //         }
            byte[][] derDecode = BCUtils.sm2SignDerDecode(sign);
            assert derDecode != null && derDecode.length == 2;
            r = derDecode[0];
            s = derDecode[1];
        } catch (Exception e) {
            throw new IOException(e);
        }
        Signature signature = new Signature();
        EccCurve eccCurve = new EccCurve(EccCurve.SGD_SM2);
        signature.setEccCurve(eccCurve);
        signature.setR(r);
        signature.setS(s);
        certificate.setSignature(signature);
        ByteArrayUtils.printHexBinary(1, "Signature", signature.getEncode());
        ByteArrayUtils.printHexBinary(0, "Certificate", certificate.getEncode());

        return certificate;
    }

}

==> oer-0.0.2-20190826.025429-23-sources.jar: com/xdja/pki/itsca/oer/cert/CertificateBuilder.java <==
package com.xdja.pki.itsca.oer.cert;


import com.xdja.pki.gmssl.crypto.sdf.SdfCryptoType;
import com.xdja.pki.gmssl.crypto.sdf.SdfPrivateKey;
import com.xdja.pki.gmssl.crypto.utils.GMSSLSM2SignUtils;
import com.xdja.pki.gmssl.crypto.utils.GMSSLSM3DigestUtils;
import com.xdja.pki.itsca.oer.asn1.*;
import com.xdja.pki.itsca.oer.asn1.base.Enumerated;
import com.xdja.pki.itsca.oer.asn1.base.Null;
import com.xdja.pki.itsca.oer.cert.bean.CertHashType;
import com.xdja.pki.itsca.oer.utils.BCUtils;
import com.xdja.pki.itsca.oer.utils.ByteArrayUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.security.PrivateKey;
import java.security.PublicKey;

public class CertificateBuilder {

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    private SequenceOfItsAidList itsAidList;
    private SequenceOfItsAidSspList itsAidSspList;

    private GeographicRegion geographicRegion;
    private Time32 startTime;
    private Time32 endTime;

    public void setStartTime(Time32 startTime) {
        this.startTime = startTime;
    }

    public void setEndTime(Time32 endTime) {
        this.endTime = endTime;
    }

    public void setItsAidList(SequenceOfItsAidList itsAidList) {
        this.itsAidList = itsAidList;
    }

    public void setItsAidSspList(SequenceOfItsAidSspList itsAidSspList) {
        this.itsAidSspList = itsAidSspList;
    }

    public void setGeographicRegion(GeographicRegion geographicRegion) {
        this.geographicRegion = geographicRegion;
    }

    /**
     * 构造 四边形区域
     *
     * @param northWestLatitude  北 西 维度
     * @param northWestLongitude 北 西 经度
     * @param southEastLatitude  南 东 纬度
     * @param southEastLongitude 南 东 经度
     * @return 四边形区域信息
     */
    public void setGeographicRegion(
            int northWestLatitude,
            int northWestLongitude,
            int southEastLatitude,
            int southEastLongitude
    ) {
        this.geographicRegion = new GeographicRegion();
        SequenceOfRectangularRegion sequenceOfRectangularRegion = new SequenceOfRectangularRegion();
        RectangularRegionBuilder rectangularRegionBuilder = new RectangularRegionBuilder();
        RectangularRegion rectangularRegion = rectangularRegionBuilder.build(northWestLatitude, northWestLongitude, southEastLatitude, southEastLongitude);
        sequenceOfRectangularRegion.addRectangularRegion(rectangularRegion);
        this.geographicRegion.setRectangularRegion(sequenceOfRectangularRegion);
    }

    public Certificate build(PrivateKey privateKey, PublicKey verifyPublicKey, PublicKey encPublicKey, String subjectName, byte[] issuedHashId, Enumerated.Value subjectTypeValue) throws Exception {
        return build(privateKey, verifyPublicKey, encPublicKey, subjectName, CertHashType.SGD_SM3, issuedHashId, subjectTypeValue);
    }

    public Certificate build(PrivateKey privateKey, PublicKey verifyPublicKey, PublicKey encPublicKey, String subjectName, CertHashType hashType, byte[] issuedHashId, Enumerated.Value subjectTypeValue) throws Exception {

        TbsCertBuilder tbsCertBuilder = new TbsCertBuilder();
        if (null != endTime) {
            tbsCertBuilder.setEndTime(endTime);
        }
        if (null != startTime) {
            tbsCertBuilder.setStartTime(startTime);
        }
        if (itsAidList != null) {
            tbsCertBuilder.setItsAidList(itsAidList);
        }
        if (itsAidSspList != null) {
            tbsCertBuilder.setItsAidSspList(itsAidSspList);
        }
        if (geographicRegion != null) {
            tbsCertBuilder.setGeographicRegion(geographicRegion);
        }

        TbsCert tbsCert = tbsCertBuilder.build(verifyPublicKey, encPublicKey, subjectName, subjectTypeValue);

        return build(privateKey, hashType, issuedHashId, tbsCert);
    }

    public Certificate build(PrivateKey privateKey, CertHashType hashType, byte[] issueHash, TbsCert tbsCert) throws Exception {
        Certificate certificate = new Certificate();
        ByteArrayUtils.printHexBinary(0, "Certificate SEQUENCE", null);

        ByteArrayUtils.printHexBinary(1, "Version Uint8 INTEGER", null);
        ByteArrayUtils.printHexBinary(1, "Version", ByteBuffer.allocate(2).putShort((short) 2).array());

        IssuerId issuerId = new IssuerId();
        if (issueHash != null) {
            ByteArrayUtils.printHexBinary(1, "IssuerId CHOICE [index = 1]", null);

            ByteArrayUtils.printHexBinary(2, "CertificateDigest SEQUENCE", null);
            CertificateDigest certificateDigest = new CertificateDigest();

            ByteArrayUtils.printHexBinary(3, "HashAlgorithm ENUMERATED", null);
            HashAlgorithm hashAlgorithm;
            if (hashType.id == CertHashType.SGD_SM3.id) {
                hashAlgorithm = new HashAlgorithm(HashAlgorithm.SGD_SM3);
            } else {
                hashAlgorithm = new HashAlgorithm(HashAlgorithm.SHA_256);
            }

            certificateDigest.setHashAlgorithm(hashAlgorithm);
            ByteArrayUtils.printHexBinary(3, "HashAlgorithm", hashAlgorithm.getEncode());

            ByteArrayUtils.printHexBinary(3, "HashedId8 OCTER STRING", null);
            byte[] hashId;
            try {
                hashId = ByteArrayUtils.subByteHashId(issueHash, 8);
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
            HashedId8 hashedId8 = new HashedId8(hashId);
            certificateDigest.setHashedId8(hashedId8);
            ByteArrayUtils.printHexBinary(3, "HashedId8", hashedId8.getEncode());

            ByteArrayUtils.printHexBinary(2, "CertificateDigest", certificateDigest.getEncode());
            issuerId.setCertificateDigest(certificateDigest);
        } else {
            ByteArrayUtils.printHexBinary(1, "IssuerId CHOICE [index = 0]", null);
            Null aNull = new Null();
            issuerId.setSelf(aNull);
        }
        certificate.setIssuerId(issuerId);
        ByteArrayUtils.printHexBinary(1, "IssuerId", issuerId.getEncode());

        ByteArrayUtils.printHexBinary(1, "TbsCert", null);
        certificate.setTbsCert(tbsCert);
        ByteArrayUtils.printHexBinary(1, "TbsCert", tbsCert.getEncode());

        byte[] tbsEncode = tbsCert.getEncode();
        byte[] data;
        if (issueHash != null) {
            data = ByteArrayUtils.buildUpByte(GMSSLSM3DigestUtils.digestByYunhsm(tbsEncode), issueHash);
        } else {
            data = ByteArrayUtils.buildUpByte(GMSSLSM3DigestUtils.digestByYunhsm(tbsEncode),
                    GMSSLSM3DigestUtils.digestByYunhsm("".getBytes()));
        }
        byte[] r;
        byte[] s;
        try {
            byte[] sign;
            if (privateKey instanceof SdfPrivateKey) {
                sign = GMSSLSM2SignUtils.signBySdfWithUserId(SdfCryptoType.YUNHSM,
                        ((SdfPrivateKey) privateKey).getIndex(),
                        new String(((SdfPrivateKey) privateKey).getPassword()),
                        "China".getBytes(),
                        data);
            } else {
                sign = BCUtils.sm2SignByBC(privateKey, data);
            }
            byte[][] derDecode = BCUtils.sm2SignDerDecode(sign);
            assert derDecode != null && derDecode.length == 2;
            r = derDecode[0];
            s = derDecode[1];
        } catch (Exception e) {
            throw new Exception(e);
        }
        Signature signature = new Signature();
        EccCurve eccCurve = new EccCurve(EccCurve.SGD_SM2);
        signature.setEccCurve(eccCurve);
        signature.setR(r);
        signature.setS(s);
        certificate.setSignature(signature);
        ByteArrayUtils.printHexBinary(1, "Signature", signature.getEncode());
        ByteArrayUtils.printHexBinary(0, "Certificate", certificate.getEncode());

        return certificate;
    }

}
